""" Commands to gather information related to device inventory in DNAC """

import click
from rich.table import Table
from rich.console import Console
import requests


@click.command
@click.option(
    "--hostname",
    default="",
    help="Specify a device's hostname to retrieve from inventory",
)
@click.pass_context
def devices(ctx, hostname):
    """Retrieve all devices from DNAC inventory"""
    headers = {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "X-Auth-Token": ctx.obj.token,
    }
    if not ctx.obj.dnac_url:
        raise click.ClickException(
            "DNAC URL has not been provided and has not been set as an environment variable."
        )
    if hostname:
        dnac_devices_url = (
            f"{ctx.obj.dnac_url}/dna/intent/api/v1/network-device?hostname={hostname}"
        )
    else:
        dnac_devices_url = f"{ctx.obj.dnac_url}/dna/intent/api/v1/network-device"
    response = requests.get(url=dnac_devices_url, headers=headers, verify=False)
    if response.status_code == 200:
        device_list = response.json()["response"]
        table = Table(title="DNAC Network Devices")
        table.add_column("Hostname", justify="left", style="purple")
        table.add_column("Device Type", justify="left", style="cyan")
        table.add_column("Serial Number", justify="center", style="green")
        table.add_column("Software Version", justify="right", style="red")

        for device in device_list:
            table.add_row(
                device["hostname"],
                device["type"],
                device["serialNumber"],
                device["softwareVersion"],
            )

        console = Console()
        console.print(table)
        # click.echo(f"Here's a list of devices: {console.print(table)}")
    elif response.status_code == 401:
        click.echo("Unauthorized. Please verify your token is valid.")
        click.echo(response.text)
    else:
        click.echo("Could not retrieve list of network devices from DNAC.")



#!/usr/bin/env python

"""DNAC Sidekick
A simple CLI app built on the Click library to interact with Cisco DNA Center.
Here are the current features/functions of the tool:
    * Get device information for all of DNAC's inventory or just a specific device
      (via hostname)
    * Get device health statistics for all devices
    * Get client health statistics for all clients or just a specific client
      (via the client's MAC address)
    * Run arbitrary "show" commands against a specific device in DNAC's inventory
Use --help to check available commands and options.
Feel free to open a PR or fork the project to modify to your needs.
"""

import click
import requests
from requests.auth import HTTPBasicAuth
import os
from dotenv import load_dotenv

from dnac_sidekick.inventory import commands as inventory_cmds
from dnac_sidekick.health import commands as health_cmds
from dnac_sidekick.device_commands import commands as device_commands_cmds
from dnac_sidekick.licenses import commands as license_cmds

dotenv_file = "../.env"
load_dotenv(dotenv_file)
requests.packages.urllib3.disable_warnings()


class DnacUser(object):
    def __init__(self, dnac_url=None, username=None, password=None, token=None):
        self.dnac_url = dnac_url
        self.dnac_user = username
        self.dnac_pass = password
        self.token = token


@click.group()
@click.pass_context
def dnac_cli(ctx):
    """
    Extract sensitive info from environment variables that will be used to connect to DNA Center and add to Click Context.
    By adding to Click Context, these values can be used across all commands.
    """
    pass


@dnac_cli.command
def login():
    """Helper function to quickly generate bearer token and authenticate to DNAC."""
    # Pull env vars that should be set by user
    dnac_url = os.environ.get("DNAC_URL")
    username = os.environ.get("DNAC_USER")
    password = os.environ.get("DNAC_PASS")
    # Confirm set env var values are not None
    if None in (dnac_url, username, password):
        raise click.ClickException("A necessary environment variable has not been set.")
    # Since value is being read from env var, and not context, need to add extra logic
    if dnac_url[-1] == "/":
        dnac_url = dnac_url.rstrip(dnac_url[-1])
    headers = {
        "Content-Type": "application/json",
        "Accept": "application/json",
    }
    dnac_token_url = f"{dnac_url}/dna/system/api/v1/auth/token"
    click.echo("Attempting to login to DNAC...")
    token = requests.post(
        url=dnac_token_url,
        headers=headers,
        auth=HTTPBasicAuth(username=username, password=password),
        verify=False,
    )
    if token.status_code == 200:
        actual_token = token.json()["Token"]
        click.echo("Token generated successfully!")
        # Set new token as env var and update .env file
        click.echo(
            "Copy token below and set as environment variable for future requests:"
        )
        click.echo(actual_token)

    elif token.status_code == 401:
        click.echo(
            "Unauthorized. Token not generated. Please make sure a proper username and password are provided and correct."
        )
    else:
        click.echo("Token not generated. Please try again...")
        click.echo(f"Status code: {token.status_code}. Error message: {token.text}.")


@dnac_cli.group()
@click.pass_context
def get(ctx):
    """Action for read-only tasks and gathering information."""
    click.echo("Getting information...")

    # Confirm all the necessary env vars are set
    dnac_url = os.environ.get("DNAC_URL")
    dnac_user = os.environ.get("DNAC_USER")
    dnac_pass = os.environ.get("DNAC_PASS")
    dnac_token = os.environ.get("DNAC_TOKEN")
    # Add values to context for read-only actions to use
    if None not in (dnac_url, dnac_user, dnac_pass, dnac_token):
        ctx.obj = DnacUser(
            dnac_url=dnac_url,
            username=dnac_user,
            password=dnac_pass,
            token=dnac_token,
        )
    else:
        raise click.ClickException("A necessary environment variable has not been set.")


@get.group()
@click.pass_context
def inventory(ctx):
    """Gathers information related to device inventory in DNAC"""
    pass


@get.group()
@click.pass_context
def health(ctx):
    """Gathers health information for network devices and clients in DNAC"""
    pass


inventory.add_command(inventory_cmds.devices)
health.add_command(health_cmds.devices)
health.add_command(health_cmds.clients)
dnac_cli.add_command(device_commands_cmds.command_runner)
get.add_command(license_cmds.licenses)

if __name__ == "__main__":
    dnac_cli()



""" Commands to run CLI commands on network devices in DNAC inventory and view the output. """

import click
from rich.table import Table
from rich.console import Console
import requests


@click.command
@click.option(
    "--device", default="", help="Specify a device's hostname to get licensing."
)
@click.pass_context
def licenses(ctx, device):
    """Get license info for devices in DNAC inventory."""
    headers = {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "X-Auth-Token": ctx.obj.token,
    }
    if not ctx.obj.dnac_url:
        raise click.ClickException(
            "DNAC URL has not been provided or has not been set as an environment variable."
        )
    # different workflow if a device hostname is specified
    if device:
        dnac_devices_url = (
            f"{ctx.obj.dnac_url}/dna/intent/api/v1/network-device?hostname={device}"
        )
        net_devices_resp = requests.get(
            url=dnac_devices_url, headers=headers, verify=False
        )
        if net_devices_resp.status_code == 200:
            dev_id = net_devices_resp.json()["response"][0].get("id")
            # raise exception if device ID is not found
            if not dev_id:
                raise click.ClickException("Device hostname not found in inventory.")
        # could handle more error codes (i.e. 400, 401, etc) instead of using else
        else:
            raise click.ClickException(
                f"Could not pull device ID. Status code: {net_devices_resp.status_code}. Error message: {net_devices_resp.text}"
            )
        dev_licensing_url = (
            f"{ctx.obj.dnac_url}/dna/intent/api/v1/licenses/device/{dev_id}/details"
        )
        dev_licensing_resp = requests.get(
            url=dev_licensing_url, headers=headers, verify=False
        )
        if dev_licensing_resp.status_code == 200:
            device_lic_details = dev_licensing_resp.json()
            table = Table(title="DNAC Network Device Licensing")
            table.add_column(
                "Network License Level", justify="left", style="turquoise2"
            )
            table.add_column("DNA License Level", justify="left", style="purple")
            table.add_column("License Validity", justify="center", style="cyan")
            table.add_column("Virtual Account", justify="center", style="green")
            table.add_column("Device UDI", justify="center", style="red")

            if device_lic_details.get("is_license_expired") == False:
                lic_validity = "[bold green3]Valid[bold green3]"
            else:
                lic_validity = "[bold red]Expired[/bold red]"

            table.add_row(
                device_lic_details.get("network_license", "N/A"),
                device_lic_details.get("dna_level", "N/A"),
                lic_validity,
                device_lic_details.get("virtual_account_name", "N/A"),
                device_lic_details.get("udi", "N/A"),
            )

            console = Console()
            console.print(table)
        elif dev_licensing_resp.status_code == 401:
            click.echo("Unauthorized. Please verify your token is valid.")
            click.echo(dev_licensing_resp.text)
        else:
            click.echo("Could not retrieve license status of network device from DNAC.")
    else:
        dnac_devices_url = f"{ctx.obj.dnac_url}/dna/intent/api/v1/network-device"
        net_devices_resp = requests.get(
            url=dnac_devices_url, headers=headers, verify=False
        )
        if net_devices_resp.status_code == 200:
            net_devs = net_devices_resp.json()["response"]
            dev_ids = []
            for dev in net_devs:
                dev_ids.append(dev.get("id"))
            # raise exception if device ID list is empty
            if not dev_ids:
                raise click.ClickException("Device IDs could not be found.")
        elif dev_licensing_resp.status_code == 401:
            click.echo("Unauthorized. Please verify your token is valid.")
            click.echo(dev_licensing_resp.text)
        else:
            click.echo(
                "Could not retrieve license status of network device(s) from DNAC."
            )
        # initialize table to pretty print data
        table = Table(title="DNAC Network Device Licensing")
        table.add_column("Network License Level", justify="left", style="blue")
        table.add_column("DNA License Level", justify="left", style="purple")
        table.add_column("License Validity", justify="center", style="cyan")
        table.add_column("Virtual Account", justify="center", style="green")
        table.add_column("Device UDI", justify="center", style="red")
        # Pull license data for each device found in inventory
        for net_id in dev_ids:
            dev_licensing_url = (
                f"{ctx.obj.dnac_url}/dna/intent/api/v1/licenses/device/{net_id}/details"
            )
            dev_licensing_resp = requests.get(
                url=dev_licensing_url, headers=headers, verify=False
            )
            if dev_licensing_resp.status_code == 200:
                device_lic_details = dev_licensing_resp.json()

                if device_lic_details.get("is_license_expired") == False:
                    lic_validity = "[bold green3]Valid[bold green3]"
                else:
                    lic_validity = "[bold red]Expired[/bold red]"

                table.add_row(
                    device_lic_details.get("network_license", "N/A"),
                    device_lic_details.get("dna_level", "N/A"),
                    lic_validity,
                    device_lic_details.get("virtual_account_name", "N/A"),
                    device_lic_details.get("udi", "N/A"),
                )
            elif dev_licensing_resp.status_code == 401:
                click.echo("Unauthorized. Please verify your token is valid.")
                click.echo(dev_licensing_resp.text)
            else:
                click.echo(
                    "Could not retrieve license status of network device(s) from DNAC."
                )

        console = Console()
        console.print(table)



""" Commands to gather health information for network devices and clients in DNAC """

import click
from rich.table import Table
from rich.console import Console
import requests


@click.command()
@click.pass_context
def devices(ctx):
    """Retrieve device health for all devices in DNAC inventory"""
    headers = {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "X-Auth-Token": ctx.obj.token,
    }
    dnac_device_health = f"{ctx.obj.dnac_url}/dna/intent/api/v1/device-health"
    response = requests.get(url=dnac_device_health, headers=headers, verify=False)
    if response.status_code == 200:
        device_list = response.json()["response"]
        table = Table(title="DNAC Network Health")
        table.add_column("Hostname", justify="left", style="purple")
        table.add_column("Overall Health", justify="left", style="cyan")
        table.add_column("CPU Util (%)", justify="center", style="green")
        table.add_column("Memory Util (%)", justify="right", style="red")

        for device in device_list:
            if device.get("overallHealth", -1) < 0:
                device_overall = "N/A"
            else:
                device_overall = str(device.get("overallHealth", "N/A"))
            cpu_util = round(device.get("cpuUlitilization", 0), 1)
            device_cpu_util = str(cpu_util)
            mem_util = round(device.get("memoryUtilization", 0), 1)
            device_mem_util = str(mem_util)
            table.add_row(
                device["name"], device_overall, device_cpu_util, device_mem_util
            )

        console = Console()
        console.print(table)
    elif response.status_code == 401:
        click.echo("Unauthorized. Please verify your token is valid.")
    else:
        click.echo("Could not retrieve list of network devices from DNAC.")


@click.command()
@click.pass_context
def clients(ctx):
    """Retrieve client health for all tracked clients in DNAC"""
    headers = {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "X-Auth-Token": ctx.obj.token,
    }
    dnac_client_health = f"{ctx.obj.dnac_url}/dna/intent/api/v1/client-health"
    response = requests.get(url=dnac_client_health, headers=headers, verify=False)
    if response.status_code == 200:
        device_list = response.json()["response"]
        table = Table(title="DNAC Client Health")
        table.add_column("Client Type", justify="left", style="purple")
        table.add_column("Client Count", justify="center", style="cyan")
        table.add_column("Client Health Score", justify="right", style="green")

        for device in device_list:
            if device.get("siteId") == "global":
                for score in device.get("scoreDetail"):
                    client_type = score["scoreCategory"]["value"]
                    client_count = (
                        str(score["clientCount"]) if score["clientCount"] > 0 else "0"
                    )
                    client_score = (
                        str(score["scoreValue"]) if score["scoreValue"] > 0 else "0"
                    )

                    table.add_row(client_type, client_count, client_score)

        console = Console()
        console.print(table)
    elif response.status_code == 401:
        click.echo("Unauthorized. Please verify your token is valid.")
    else:
        click.echo("Could not retrieve list of network devices from DNAC.")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dnac_sidekick',
 'dnac_sidekick.device_commands',
 'dnac_sidekick.health',
 'dnac_sidekick.inventory',
 'dnac_sidekick.licenses']

package_data = \
{'': ['*']}

install_requires = \
['Pygments==2.13.0',
 'attrs==22.1.0',
 'black==22.8.0',
 'certifi==2022.6.15.1',
 'charset-normalizer==2.1.1',
 'click==8.1.3',
 'commonmark==0.9.1',
 'coverage==6.4.4',
 'idna==3.3',
 'iniconfig==1.1.1',
 'mypy-extensions==0.4.3',
 'packaging==21.3',
 'pathspec==0.10.1',
 'platformdirs==2.5.2',
 'pluggy==1.0.0',
 'py==1.11.0',
 'pyparsing==3.0.9',
 'pytest-cov==3.0.0',
 'pytest==7.1.3',
 'python-dotenv==0.21.0',
 'requests==2.28.1',
 'rich==12.5.1',
 'tomli==2.0.1',
 'typing-extensions==4.3.0',
 'urllib3==1.26.12']

entry_points = \
{'console_scripts': ['dnac-sidekick = dnac_sidekick.cli:dnac_cli']}

setup_kwargs = {
    'name': 'dnac-sidekick',
    'version': '0.1.1',
    'description': 'A CLI app used to interact with Cisco DNA Center',
    'long_description': '[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![codecov](https://codecov.io/gh/dannywade/dnac-sidekick/branch/main/graph/badge.svg?token=IWBEDN1YXH)](https://codecov.io/gh/dannywade/dnac-sidekick)\n\n# dnac-sidekick\nDNAC Sidekick is a CLI app used to interact with Cisco DNA Center (DNAC). It\'s built using the [Click](https://github.com/pallets/click) and [Rich](https://github.com/Textualize/rich) libraries. The Rich library is what helps make the output look cleaner to the end-user. \n\nThe goal of the tool is to provide a clean and user-friendly CLI interface to quickly pull information from DNA Center. All interactions with DNAC uses DNA Center\'s REST API, so please make sure that the REST API is enabled and the user account(s) used with this tool have proper API permissions.\n\n## Installation\nInstall using `pip` or any other PyPi package manager:\n```\npip install dnac-sidekick\n```\n\n## Getting Started\n\n### Authenticating to DNAC\nDNAC-Sidekick pulls all user-specific information from environmnet variables. Ideally, this would be a more automated process with less manual work for the user, but for now, the following environment variables must be set manually before using the tool:\n```\nUnix/MacOS\nexport DNAC_URL=<https://dnac_url>\nexport DNAC_USER=<username>\nexport DNAC_PASS=<password>\n\nWindows\nset DNAC_URL=<https://dnac_url>\nset DNAC_USER=<username>\nset DNAC_PASS=<password>\n```\n\nOnce set, we will need to generate a bearer token, which is used to authenticate to the DNAC REST API. You can manually generate this token using curl or Postman, but there\'s also a built-in command that will generate one for you. This will only work if the URL, username, and password environment variables are set.\n\n```\ndnac-sidekick login \n\nToken generated successfully!\nCopy token below and set as environment variable for future requests:\neyJhbGciOiJS.....\n```\n\n*IMPORTANT:* Please make sure to generate the bearer token using the `dnac-sidekick login` command *AFTER* setting the necessary environment variables. Once the token is generated, don\'t forget to set it as an evironment variable as well.\n\n```\nUnix/MacOS\nexport DNAC_TOKEN=<token>\n\nWindows\nset DNAC_TOKEN=<token>\n```\n\n## Usage\nTo see what commands are available, use the `--help` option. Here\'s a brief look at the current root commands available:\n```\nOptions:\n  --help  Show this message and exit.\n\nCommands:\n  command-runner  Run \'show\' commands on network devices in DNAC.\n  get             Action for read-only tasks and gathering information.\n  login           Use username and password to authenticate to DNAC.\n```\n\n## Feature Highlights\nThe goal is to provide features that help extract the most useful information from DNAC for the user. The tool is not built to have a command for every available DNAC API call - it\'s simply meant to be an engineer\'s *sidekick* :grin: when interacting with Cisco DNA Center. This feature list will grow, but here are the current tasks that can be performed using DNAC Sidekick:\n\n**Inventory**\n- Device information (all devices or a specific device via hostname)\n  - Hostname\n  - Device type\n  - Serial number\n  - Software version\n- Device license information\n  - Network license level\n  - DNA license level\n  - License validity (valid or expired)\n  - Virtual account\n  - Device UDI\n\n**Assurance**\n- Device health\n- Client health\n  - All\n  - Wired\n  - Wireless\n\n**Command Runner**\n- Run *valid* `show` commands on any device in DNAC inventory\n  - Valid `show` commands are dictated by DNAC\n\n## Examples:\n**Inventory** \n```\n# All devices\ndnac-sidekick get inventory devices\n\n# Specific device\ndnac-sidekick get inventory devices --hostname leaf1.abc.inc\n\n# License info\ndnac-sidekick get licenses\n```\n\n**Assurance** \n```\n# Device health\ndnac-sidekick get health devices\n\n# Client health\ndnac-sidekick get health clients\n```\n\n**Command Runner** \n```\ndnac-sidekick command-runner --device leaf1.abc.inc --command "show run"\n```\n\n<details>\n<summary>Sample Outputs</summary>\n\n***All sample outputs use the Cisco DevNet Always-on DNAC sandbox.***\n\n### Network Inventory\n![Network Inventory](./imgs/get_network_inventory.png)\n\n### Network Inventory - Specific Device\n![Inventory - Specific Device](./imgs/get_specific_device.png)\n\n### Device Licensing\n![Device Licensing](./imgs/get_device_licensing.png)\n\n### Device Health\n![Device Health](./imgs/get_device_health.png)\n\n### Client Health\n![Client Health](./imgs/get_client_health.png)\n\n</details>\n<br>\n\n## Compatibility\nTested with:\n- DNA Center 2.2.3.4\n\n*If you are able to test with other versions, please open a PR and add it to the list!*\n',
    'author': 'Dan Wade',
    'author_email': 'danny.wade35@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/dannywade/dnac-sidekick',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



""" Commands to run CLI commands on network devices in DNAC inventory and view the output. """

import click
import requests
import json
import os


@click.command
@click.option(
    "--device", required=True, help="Specify a device's hostname to run commands."
)
@click.option(
    "--command", required=True, help="Specify a command to run on the specified device."
)
def command_runner(device, command):
    """Run 'show' commands on network devices in DNAC."""
    # Confirm all the necessary env vars are set
    dnac_url = os.environ.get("DNAC_URL")
    dnac_user = os.environ.get("DNAC_USER")
    dnac_pass = os.environ.get("DNAC_PASS")
    dnac_token = os.environ.get("DNAC_TOKEN")
    # Add values to context for read-only actions to use
    if None in (dnac_url, dnac_user, dnac_pass, dnac_token):
        raise click.ClickException("A necessary environment variable has not been set.")
    headers = {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "X-Auth-Token": dnac_token,
    }
    dnac_devices_url = f"{dnac_url}/dna/intent/api/v1/network-device?hostname={device}"
    net_devices_resp = requests.get(url=dnac_devices_url, headers=headers, verify=False)
    if net_devices_resp.status_code == 200:
        dev_id = net_devices_resp.json()["response"][0].get("id")
        if not dev_id:
            raise click.ClickException("Device hostname not found in inventory.")
    dnac_command_run = (
        f"{dnac_url}/dna/intent/api/v1/network-device-poller/cli/read-request"
    )
    payload = {
        "timeout": 5,
        "description": "Just a simple command ran by DNAC sidekick",
        "name": "DNAC sidekick command run",
        "commands": [command],
        "deviceUuids": [dev_id],
    }
    # Run command, which kicks off a task in DNAC
    comm_run_resp = requests.post(
        url=dnac_command_run, headers=headers, json=payload, verify=False
    )
    if comm_run_resp.status_code == 202:
        task_id = comm_run_resp.json()["response"].get("taskId")
        if not task_id:
            raise click.ClickException("Task ID not found.")
    else:
        print("Could not get task ID.")
        print(f"Status code: {comm_run_resp.status_code}")
        print(f"Error: {comm_run_resp.text}")
    # Get task by ID
    task_check = f"{dnac_url}/api/v1/task/{task_id}"
    task_check_resp = requests.get(url=task_check, headers=headers, verify=False)
    if task_check_resp.status_code == 200:
        tasks_found = task_check_resp.json()["response"]
        task_end = tasks_found.get("endTime")
        while task_end is None:
            task_checkup = requests.get(url=task_check, headers=headers, verify=False)
            resp = task_checkup.json()["response"]
            end_time = resp.get("endTime")
            if end_time is not None:
                raw_file_id = json.loads(resp.get("progress"))
                file_id = raw_file_id.get("fileId")
                task_end = 0
    else:
        print(f"Status code: {task_check_resp.status_code}")
        print(f"Error! Error message: {task_check_resp.text}")
        raise click.ClickException("File ID not found.")
    # Get file by ID
    dnac_get_file = f"{dnac_url}/dna/intent/api/v1/file/{file_id}"
    file_resp = requests.get(url=dnac_get_file, headers=headers, verify=False)
    if net_devices_resp.status_code == 200:
        command_output = file_resp.json()[0]["commandResponses"]["SUCCESS"][command]
        print(command_output)
